NCBI_HOST = 'https://www.ncbi.nlm.nih.gov'
SERIES_URL = f'{NCBI_HOST}/gds/'
SERIES_DETAIL_URL = f'{NCBI_HOST}/geo/query/acc.cgi?acc='


chrome_ver = 124
USER_AGENT = (
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
    'AppleWebKit/537.36 (KHTML, like Gecko) '
    f'Chrome/{chrome_ver}.0.0.0 '
    'Safari/537.36'
)
